@@ -11,9 +11,6 @@ local tsort,append,remove = table.sort,table.insert,table.remove
 local min,max = math.min,math.max
 local pairs,type,unpack,next,select,tostring = pairs,type,utils.unpack,next,select,tostring
 local function_arg = utils.function_arg
-local Set = utils.stdmt.Set
-local List = utils.stdmt.List
-local Map = utils.stdmt.Map
 local assert_arg = utils.assert_arg
 
 local tablex = {}
@@ -21,13 +18,17 @@ local tablex = {}
 -- generally, functions that make copies of tables try to preserve the metatable.
 -- However, when the source has no obvious type, then we attach appropriate metatables
 -- like List, Map, etc to the result.
-local function setmeta (res,tbl,def)
-    local mt = getmetatable(tbl) or def
-    return setmetatable(res, mt)
+local function setmeta (res,tbl,pl_class)
+    local mt = getmetatable(tbl) or pl_class and require('pl.' .. pl_class)
+    return mt and setmetatable(res, mt) or res
 end
 
-local function makelist (res)
-    return setmetatable(res,List)
+local function makelist(l)
+    return setmetatable(l, require('pl.List'))
+end
+
+local function makemap(m)
+    return setmetatable(m, require('pl.Map'))
 end
 
 local function complain (idx,msg)
@@ -267,7 +268,7 @@ function tablex.index_by(tbl,idx)
     for i = 1,#idx do
         res[i] = tbl[idx[i]]
     end
-    return setmeta(res,tbl,List)
+    return setmeta(res,tbl,'List')
 end
 
 --- apply a function to all values of a table.
@@ -304,7 +305,7 @@ function tablex.imap(fun,t,...)
     for i = 1,#t do
         res[i] = fun(t[i],...) or false
     end
-    return setmeta(res,t,List)
+    return setmeta(res,t,'List')
 end
 
 --- apply a named method to values from a table.
@@ -321,7 +322,7 @@ function tablex.map_named_method (name,t,...)
         local fun = val[name]
         res[i] = fun(val,...)
     end
-    return setmeta(res,t,List)
+    return setmeta(res,t,'List')
 end
 
 --- apply a function to all values of a table, in-place.
@@ -373,7 +374,7 @@ function tablex.map2 (fun,t1,t2,...)
     for k,v in pairs(t1) do
         res[k] = fun(v,t2[k],...)
     end
-    return setmeta(res,t1,List)
+    return setmeta(res,t1,'List')
 end
 
 --- apply a function to values from two arrays.
@@ -473,8 +474,8 @@ end
 
 --- call the function with the key and value pairs from a table.
 -- The function can return a value and a key (note the order!). If both
--- are not nil, then this pair is inserted into the result. If only value is not nil, then
--- it is appended to the result.
+-- are not nil, then this pair is inserted into the result: if the key already exists, we convert the value for that
+-- key into a table and append into it. If only value is not nil, then it is appended to the result.
 -- @within MappingAndFiltering
 -- @func fun A function which will be passed each key and value as arguments, plus any extra arguments to pairmap.
 -- @tab t A table
@@ -488,7 +489,15 @@ function tablex.pairmap(fun,t,...)
     for k,v in pairs(t) do
         local rv,rk = fun(k,v,...)
         if rk then
-            res[rk] = rv
+			if res[rk] then
+				if type(res[rk]) == 'table' then
+					table.insert(res[rk],rv)
+				else
+					res[rk] = {res[rk], rv}
+				end
+			else
+            	res[rk] = rv
+			end
         else
             res[#res+1] = rv
         end
@@ -524,7 +533,7 @@ local function index_map_op (i,v) return i,v end
 -- @return a map-like table
 function tablex.index_map (t)
     assert_arg_indexable(1,t)
-    return setmetatable(tablex.pairmap(index_map_op,t),Map)
+    return makemap(tablex.pairmap(index_map_op,t))
 end
 
 local function set_op(i,v) return true,v end
@@ -535,7 +544,7 @@ local function set_op(i,v) return true,v end
 -- @return a set (a map-like table)
 function tablex.makeset (t)
     assert_arg_indexable(1,t)
-    return setmetatable(tablex.pairmap(set_op,t),Set)
+    return setmetatable(tablex.pairmap(set_op,t),require('pl.Set'))
 end
 
 --- combine two tables, either as union or intersection. Corresponds to
@@ -560,7 +569,26 @@ function tablex.merge (t1,t2,dup)
         res[k] = v
       end
     end
-    return setmeta(res,t1,Map)
+    return setmeta(res,t1,'Map')
+end
+
+--- the union of two map-like tables.
+-- If there are duplicate keys, the second table wins.
+-- @tab t1 a table
+-- @tab t2 a table
+-- @treturn tab
+-- @see tablex.merge
+function tablex.union(t1, t2)
+    return tablex.merge(t1, t2, true)
+end
+
+--- the intersection of two map-like tables.
+-- @tab t1 a table
+-- @tab t2 a table
+-- @treturn tab
+-- @see tablex.merge
+function tablex.intersection(t1, t2)
+    return tablex.merge(t1, t2, false)
 end
 
 --- a new table which is the difference of two tables.
@@ -583,7 +611,7 @@ function tablex.difference (s1,s2,symm)
             if s1[k] == nil then res[k] = v end
         end
     end
-    return setmeta(res,s1,Map)
+    return setmeta(res,s1,'Map')
 end
 
 --- A table where the key/values are the values and value counts of the table.
@@ -594,7 +622,7 @@ end
 function tablex.count_map (t,cmp)
     assert_arg_indexable(1,t)
     local res,mask = {},{}
-    cmp = function_arg(2,cmp)
+    cmp = function_arg(2,cmp or '==')
     local n = #t
     for i = 1,#t do
         local v = t[i]
@@ -604,12 +632,7 @@ function tablex.count_map (t,cmp)
             res[v] = 1  -- there's at least one instance
             for j = i+1,n do
                 local w = t[j]
-                local ok
-                if cmp then
-                    ok = cmp(v,w)
-                else
-                    ok = v == w
-                end
+                local ok = cmp(v,w)
                 if ok then
                     res[v] = res[v] + 1
                     mask[w] = true
@@ -617,7 +640,7 @@ function tablex.count_map (t,cmp)
             end
         end
     end
-    return setmetatable(res,Map)
+    return makemap(res)
 end
 
 --- filter an array's values using a predicate function
@@ -636,7 +659,7 @@ function tablex.filter (t,pred,arg)
             k = k + 1
         end
     end
-    return setmeta(res,t,List)
+    return setmeta(res,t,'List')
 end
 
 --- return a table where each element is a table of the ith values of an arbitrary
@@ -725,7 +748,7 @@ function tablex.sub(t,first,last)
     first,last = tablex._normalize_slice(t,first,last)
     local res={}
     for i=first,last do append(res,t[i]) end
-    return setmeta(res,t,List)
+    return setmeta(res,t,'List')
 end
 
 --- set an array range to a value. If it's a function we use the result
@@ -869,12 +892,14 @@ end
 -- @usage for k,v in tablex.sortv(t) do print(k,v) end
 -- @return an iterator to traverse elements sorted by the values
 function tablex.sortv(t,f)
-    local rev = {}
-    for k,v in pairs(t) do rev[v] = k end
-    local next = tablex.sort(rev,f)
+    f = function_arg(2, f or '<')
+    local keys = {}
+    for k in pairs(t) do keys[#keys + 1] = k end
+    tsort(keys,function(x, y) return f(t[x], t[y]) end)
+    local i = 0
     return function()
-        local value,key = next()
-        return key,value
+        i = i + 1
+        return keys[i], t[keys[i]]
     end
 end
 
